// file: MovinGridCanvas.java
//

// import necessary java libraries
//
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.lang.Math;
import java.util.*;

// include the graph package
//
import graph.*;

/**
 * Subclass of class G2Dint which plots the animated waveform 
 * using the graph package on the output panel
 *
 * hierarchy:  G2Dint->MovinGridCanvas 
 */
public class MovinGridCanvas extends G2Dint {
     
    // *********************************************************************
    //
    // declare global variables and components
    //
    // *********************************************************************

    /**
     * boolean flag to indicate that animation has begun and the previous
     * data set that was draw as part of the animation process need to be 
     * removed from the list of data set currently being stored
     */
    boolean animation = false;

    /**
     * set up a flag to determine if the display needs to be
     * updated and initialize it to false
     */
    boolean updateFlag = false;

    /**
     * data sets for the x and y-axes
     */
    DataSet dataSet;

    // declare the axes
    //
    Axis xAxis, yAxis;

    /**
     * declare an index that keeps track of the number of data point
     * that have been added so far to the curve array
     */
    int currLimit = 0;

    /**
     * declare variable to keep track of last waveform drawn
     */
    int presetWave = -1;
    
    // set the default limits for the axis
    //
    public double outXposLimit = 0.0;
    public double outXnegLimit = 0.0;
    public double outYposLimit = 0.0;  
    public double outYnegLimit = 0.0; 
    public double outXticDivision = 0.0;
    public double outYticDivision = 0.0;
    
    
    //declare variables used to initialize plot labels
    //    
    Double OUT_X_TIC = new Double((double)1.0);
    Double OUT_Y_TIC = new Double((double)0.5);
    Double OUT_X_MAX = new Double((double)10.0);
    Double OUT_X_MIN = new Double((double)-10.0);
    Double OUT_Y_MAX = new Double((double)2.0);
    Double OUT_Y_MIN = new Double((double)-2.0);

    // declare the default fonts
    //
    Font currentFont = getFont();
    Font newFont = new Font(currentFont.getName(), currentFont.getStyle(), 10);

    Data data;

    // *********************************************************************
    //
    // declare class constructors
    //
    // *********************************************************************
    
    /**
     * default constructor for drawing the animated waveforms
     */
    public MovinGridCanvas()  {
	super();
    }

    /**
     * constructor for drawing the animated waveforms using the given samples
     *
     * @param d data object containing the data samples 
     */
    public MovinGridCanvas (Data d)  {	

	super();

	data = d;

        // remove all old datasets
        //
        this.detachDataSets();

        // initialize the arguments of this class
        //
        this.drawzero = true;
        this.drawgrid = true;
        this.borderTop = 5;
        this.borderLeft = 0;
        this.borderBottom = 0;
        this.borderRight = 25;
        this.framecolor = Color.black;
        this.gridcolor = Color.lightGray;
        this.setDataBackground(Color.white);

	// attach the data to the x-axis
        //
        xAxis = this.createXAxis();
        xAxis.setTitleText(" Time (Sec) ");
        xAxis.setTitleFont(new Font("TimesRoman", Font.PLAIN,8));
        xAxis.setLabelFont(new Font("Helvetica", Font.PLAIN, 8));

	// set the default x-axis ranges and ticks
	//
        xAxis.setXMin(OUT_X_MIN.doubleValue());
        xAxis.setXMax(OUT_X_MAX.doubleValue());
        xAxis.setXTic(OUT_X_TIC.doubleValue());

        // create the y-axis
        //
        yAxis = this.createYAxis();
        yAxis.setTitleText(" Amp (V) ");
        yAxis.setTitleFont(new Font("TimesRoman", Font.PLAIN, 8));
        yAxis.setLabelFont(new Font("Helvetica", Font.PLAIN, 8));

	// set the default y-axis ranges and ticks
	//
        yAxis.setYMin(OUT_Y_MIN.doubleValue());
        yAxis.setYMax(OUT_Y_MAX.doubleValue());
        yAxis.setYTic(OUT_Y_TIC.doubleValue());

        // attach the dataSet to the graph
        //
	double dummyArray[] = new double[10];
        dataSet = this.loadDataSet(dummyArray, 5);

        // attach the data sets the both axis
        //
        xAxis.attachDataSet(dataSet);
	xAxis.resetXRange();
        yAxis.attachDataSet(dataSet);
	yAxis.resetYRange();

	// set the preferred size of the input canvas
	//
	setPreferredSize(new Dimension(220, 35));
    }

    // *********************************************************************
    //
    //  declare class methods
    //
    // *********************************************************************


    /**
     * method sets the update flag causing a repaint of the canvas
     *
     * @return true
     */
    public boolean updateScreen() {
	
	// draw the output response
	//
	if (animation) {
	    this.detachDataSet(dataSet);
	}
	this.drawResponse();
	repaint();
	
	// exit gracefully
	//
	return true;
    }

    /**
     * this method draws the output response of the signal on the screen
     *
     * @return true
     */
    public boolean drawResponse() {

	//  make sure that the curve array has memory allocated to it
	//
	if(curveArray == null) {
	    curveArray = new double[2 * (datarect.width + 1)];
	}

	// make suer that the currLimit is with in bounds
	//
	if (currLimit < 0 || currLimit > curveArray.length / 2) {
	    currLimit = 1;
	}

        // attach the dataSet to the graph
        //
        dataSet = this.loadDataSet(curveArray, currLimit);

        // attach the data sets the both axis
        //
        xAxis.attachDataSet(dataSet);
        yAxis.attachDataSet(dataSet);

        // setup properties of the dataSet
        //
        dataSet.linestyle = 1;
        dataSet.linecolor = Color.blue;
    
	// reset the current limit
	//
	currLimit = 0;

        // exit gracefully
        //
        return true;
    }

    /**
     * set the value of the x limit on the positive axis
     *
     * @param newLimit new positive limit for the x-axis
     */
    public void outSetXPosLimit(double newLimit) {
	
	xAxis.setXMax(newLimit);
    }
    
    /**
     * set the value of the y limit on the positive axis
     *
     * @param newLimit new positive limit for the y-axis
     */
    public void outSetYPosLimit(double newLimit) {
	
	yAxis.setYMax(newLimit);
    }
    
    /**
     * set the value of the x limit on the negative axis
     *
     * @param newLimit new negative limit for the x-axis
     */
    public void outSetXNegLimit(double newLimit) {
	
	xAxis.setXMin(newLimit);
    }
    
    /**
     * set the value of the y limit on the negative axis
     *
     * @param newLimit new negative limit for the y-axis
     */
    public void outSetYNegLimit(double newLimit) {
	
	yAxis.setYMin(newLimit);
    }

    /**
     * set the new value for the major ticks on the x axis
     *
     * @param newXticDiv new tick value for the x-axis
     */
    public void outSetXDivision (double newXticDiv) {
	
	xAxis.setXTic(newXticDiv);
    }
    
    /**
     * set the new value for the major ticks on the y axis
     *
     * @param newYticDiv new tick value for the y-axis
     */
    public void outSetYDivision (double newYticDiv) {
	
	yAxis.setYTic(newYticDiv);
    }
}

//
// end of file








