// file: MovingDisplayCanvas.java
//

// import necessary java libraries
//
import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.lang.*;
import java.util.*;

/** 
 * MovingDisplayCanvas implements the animation of the convolved waveform
 *
 * hierarchy: G2Dint->OutGridCanvas->DisplayCanvas->MovingDisplayCanvas
 */
public class MovingDisplayCanvas extends MovinGridCanvas {
  
    // *********************************************************************
    //
    // declare global variables and components
    //
    // *********************************************************************
    
    // declare the signal plot gui
    //
    SignalPanel signalPanel;
    SystemPanel systemPanel;

    // declare the system and signal data objects
    //
    Data system;
    Data signal;

    // the data and screen.
    //
    int Xoffset = 0;
    
    // *********************************************************************
    //
    // declare class constructors
    //
    // *********************************************************************

    /**
     * default class constructor
     */
   public MovingDisplayCanvas ()  {
	super();
    }

    /**
     * initializes the signal, system and data objects to be used 
     * later in plotting the animated convolved waveform
     *
     * @param d output data samples from the convolved waveform
     * @param sig input signal panel object
     * @param sys input system panel object
     */
    public MovingDisplayCanvas (Data d, SignalPanel sig, SystemPanel sys)  {

	super(d);

	signalPanel = sig;
	systemPanel = sys;
    }
    
    // *********************************************************************
    //
    // declare class methods
    //
    // *********************************************************************


    /**
     * method clears the drawing area
     */
    public void clearDrawingArea() {

	// initializes the variables related to custom plot drawing
	//
	initCustom();

        // set the current limit to one to tell the graph package to 
        // only draw one pixel on the screen
        //
        currLimit = 1;

	// datach all data sets currently being held
	//
	this.detachDataSets();

        // draw waveform to canvas
        //
        updateScreen();
    }


    /**
     * draws the signal wave form on the output canvas centered about zero
     *
     * @param sig data points from the input signal (x1 y1 x2 y2 ... )
     * @param sigmin minimum limit of the signal panel
     * @param sigmax maximim limit of the signal panel
     */
    public void drawWave(Data sig, double sigmin, double sigmax) {

	// declare local variables
	//
	int width = 0;
	int offset = 0;
	double delta = 0.0001;

	// initialize the system and signal data object
	//
	signal = sig;

	// detach all previously stored data sets
	//
	animation = false;
	detachDataSets();

	//  allocate memory inorder to store data points only if necessary
	//
	if(curveArray == null) {
	    curveArray = new double[2 * (datarect.width + 1)];
	}
	
	// set the minimum and maximum time limits for the x-axis
	//
	xaxis.maximum = xaxis.getXMax();
	xaxis.minimum = xaxis.getXMin();

	// zero out the curve array
	//
	for(int i = 0; i < curveArray.length; i++) {
	    curveArray[i] = 0.0;
	}

	// time instants to calculate the interpolated output
	//
	double step = (double)(signal.inputSignal[2 * (signal.numsamples-1)] -
			       signal.inputSignal[0]) / signal.numsamples;
	
	int j = 0;
	int k=0;
	
	for(double i = signal.inputSignal[0];
	    i < signal.inputSignal[2 * (signal.numsamples-1)];
	    i += step ,j++,k++) {
	    
	    curveArray[2 * j] = (double)i;
	    curveArray[2 * j + 1] = (double)signal.fftData[k];
	    currLimit++;
	}

    

	// draw the waveform to the canvas
	//     
	updateScreen();
    }
    
    /**
     *  draws the impluse response centered at the specified time interval
     *
     * @param d data points from the impulse response (x1 y1 x2 y2 ... )
     * @param center time interval the impulse response is centered on
     * @param sysmin minimum limit of the system panel
     * @param sysmax maximim limit of the system panel
     */
    public void shiftWave(Data d, double center, double sysmin, 
			  double sysmax) {

	// declare local varibles
	//
	int width = 0;
	int offset = 0;
	double delta = 0.0001;

	// initialize the system data object
	//
	system = d;

	//  allocate memory inorder to store data points only if necessary
	//
	if(curveArray == null) {
	    curveArray = new double[2 * (datarect.width + 1)];
	}

	// set the minimum and maximum time limits for the x-axis
	//
	xaxis.maximum = xaxis.getXMax();
	xaxis.minimum = xaxis.getXMin();

	// min = time minimum where the wave is actually defined
	// max = time maximum where the wave is actually defined
	//
	double min = system.systemResponse[0];
	double max = system.systemResponse[2 * (system.numsamples-1)];

	// zero out the curve array
	//
	for(int i = 0; i < curveArray.length; i++) {
	    curveArray[i] = 0.0;
	}
	
	// calculate the incremental step in the x-axis dirsction
	//
	double step =  (max - min)/ system.numsamples;

	// time instants to calculate the interpolated output
	//
	int j = 0;
	int k = 0;

	for(double i = center+min; i < center+max ;
	    i += step, j++,k++) {
	    
	    curveArray[2 * j] = (double)i;
	    curveArray[2 * j + 1] = (double)system.fftData[system.numsamples+1
							  - k];
	    currLimit++;
	}
    
	
	

	// draw the waveform to the canvas
	//     
	updateScreen();
    }   

    /**
     * method takes in a input digit (double) and rounds the input digit
     * to the number of speified significant places specified
     *
     * @param doubleNumber input digit to be rounded off
     * @param decimalPlaces number of significant to round the input digit 
     *
     * @return the rounded off input digit
     */
    public double setDecimal(double doubleNumber, int decimalPlaces) {

	// declare local variables
	//
	int combine = 0;
	int wholeNumber = 0;
	int roundedDecimal = 0;

	double pastDecimal = 0.0;
	double result = 0.0;
	
	wholeNumber = (int)doubleNumber;
	pastDecimal = ((doubleNumber - wholeNumber) * 
		       Math.pow(10,decimalPlaces));
	roundedDecimal = (int) Math.round(pastDecimal);
	combine = (int)(wholeNumber * 
			Math.pow(10,decimalPlaces)+roundedDecimal);
	result = combine / Math.pow(10,decimalPlaces);

	// return the rounded off input digit
	//
	return result;
    }
    
}

//
// end of file



















