// file: SysGridCanvas.java
//

// include the graph package
//
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.lang.Math;
import java.util.*;

// include the graph package
//
import graph.*;

/**
 * Subclass of of class G2Dint that plots the system waveform using the 
 * facilities provided by the graph package in the inpulse response panel
 *
 * hierarchy: JPanel->SysGridCanvas 
 */
public class SysGridCanvas extends G2Dint implements Runnable {

    // *********************************************************************
    //
    // declare global variables and components
    //
    // *********************************************************************
       
    /**
     * constant to define the sleep time of the thread
     * measured in milliseconds
     */
    public final static long THREAD_SLEEP_TIME = 200;

    /**
     * set up a thread to take care of updating the display
     */
    Thread phaseRespThread;

    /**
     * set up a flag to determine if the display needs to be
     * updated and initialize it to false
     */
    boolean updateFlag = false;

    /**
     * data sets for the x and y-axes
     */
    DataSet dataSet;

   
    // declare the axes
    //
    Axis xAxis, yAxis;

    /**
     * declare an index that keeps track of the number of data point
     * that have been added so far to the curve array
     */
    int currLimit = 0;

    /**
     * declare variable to keep track of last waveform drawn
     */
    int presetWave = -1;
    
    // set the default limits for the axis
    //
    public double sysXposLimit = 2.0;  
    public double sysXnegLimit = -2.0; 
    public double sysYposLimit = 2.0;  
    public double sysYnegLimit = -2.0; 
    public double sysXticDivision = 0.5;
    public double sysYticDivision = 0.5;
    
    // declare variables used to initialize plot labels
    //
    Double SYS_X_TIC = new Double((double)0.5);
    Double SYS_Y_TIC = new Double((double)0.5);
    Double SYS_X_MAX = new Double((double)2.0);
    Double SYS_X_MIN = new Double((double)-2.0);
    Double SYS_Y_MAX = new Double((double)2.0);
    Double SYS_Y_MIN = new Double((double)-2.0);

    // declare the default fonts
    //
    Font currentFont = getFont();
    Font newFont = new Font(currentFont.getName(), currentFont.getStyle(), 10);

    Data data;

    // *********************************************************************
    //
    // declare class constructors
    //
    // *********************************************************************
    
    /**
     * default constructor draws the impulse waveform on the system panel
     *
     * @param d data object containing the system samples to be plotted
     */
    public SysGridCanvas(Data d)  {
	
	super();

	data = d;

        // remove all old datasets
        //
        this.detachDataSets();

        // initialize the arguments of this class
        //
        this.drawzero = true;
        this.drawgrid = true;
        this.borderTop = 5;
        this.borderLeft = 0;
        this.borderBottom = 0;
        this.borderRight = 25;
        this.framecolor = Color.black;
        this.gridcolor = Color.lightGray;
        this.setDataBackground(Color.white);

	// attach the data to the x-axis
        //
        xAxis = this.createXAxis();
        xAxis.setTitleText(" Time (Sec) ");
        xAxis.setTitleFont(new Font("TimesRoman", Font.PLAIN,8));
        xAxis.setLabelFont(new Font("Helvetica", Font.PLAIN, 8));

	// set the default x-axis ranges and ticks
	//
        xAxis.setXMin(SYS_X_MIN.doubleValue());
        xAxis.setXMax(SYS_X_MAX.doubleValue());
        xAxis.setXTic(SYS_X_TIC.doubleValue());

        // create the y-axis
        //
        yAxis = this.createYAxis();
        yAxis.setTitleText(" Amp (V) ");
        yAxis.setTitleFont(new Font("TimesRoman", Font.PLAIN, 8));
        yAxis.setLabelFont(new Font("Helvetica", Font.PLAIN, 8));

	// set the default y-axis ranges and ticks
	//
        yAxis.setYMin(SYS_Y_MIN.doubleValue());
        yAxis.setYMax(SYS_Y_MAX.doubleValue());
        yAxis.setYTic(SYS_Y_TIC.doubleValue());

        // attach the dataSet to the graph
        //
	double dummyArray[] = new double[10];
        dataSet = this.loadDataSet(dummyArray, 5);

        // attach the data sets the both axis
        //
        xAxis.attachDataSet(dataSet);
	xAxis.resetXRange();
        yAxis.attachDataSet(dataSet);
	yAxis.resetYRange();

	// set the preferred size of the input canvas
	//
	setPreferredSize(new Dimension(220, 35));

	// initialize the controlling thread and start it
        //
        phaseRespThread = new Thread(this);
        phaseRespThread.start();
    }

    // *********************************************************************
    //
    // declare class methods
    //
    // *********************************************************************

    /**
     * set the value of the x limit on the positive axis
     *
     * @param newLimit new positive limit for the x-axis
     */
    public void sysSetXPosLimit(double newLimit) {
	
	xAxis.setXMax(newLimit);
    }
    
    /**
     *  set the value of the y limit on the positive axis
     *
     * @param newLimit new positive limit for the y-axis
     */
    public void sysSetYPosLimit(double newLimit) {
	
	yAxis.setYMax(newLimit);
    }
    
    /**
     * set the value of the x limit on the negative axis
     *
     * @param newLimit new negative limit for the x-axis
     */
    public void sysSetXNegLimit(double newLimit) {
	
	xAxis.setXMin(newLimit);
    }

    /**
     * set the value of the y limit on the negative axis
     *
     * @param newLimit new negative limit for the y-axis
     */
    public void sysSetYNegLimit(double newLimit) {
	
	yAxis.setYMin(newLimit);
    }
    
    /**
     * set the new value for the major ticks on the x axis
     *
     * @param newXticDiv new tick value for the x-axis
     */
    public void sysSetXDivision (double newXticDiv) {
	
	xAxis.setXTic(newXticDiv);
    }
    
    /**
     * set the new value for the major ticks on the y axis
     *
     * @param newYticDiv new tick value for the y-axis
     */
    public void sysSetYDivision (double newYticDiv) {
	
	yAxis.setYTic(newYticDiv);
    }

    /**
     * write curve data to data class so fft can be performed
     * set flag after data written
     */
    void writeData () {
	
	// declare local variables
	//
	int zero = -1;
	boolean gotZero = false;

	// check to make sure that there is valid data before plotting
	//
	if (curveArray == null) {
	    return;
	}
	
	// get the capacity of the input samples
	//
	setCapacity(data);

	// copy curveArray values into fft data array and multiply by the 
	// time divion for each grid
	//
	for (int i = 0; i < curveArray.length / 2; i++) {

	    // get the index location of x = 0
	    //
	    if (curveArray[2 * i] >= 0.0 && !gotZero) {
		zero = i;
		gotZero = true;
	    }

	    data.fftData[i] = curveArray[2 * i + 1];
	    data.systemResponse[2 * i] = curveArray[2 * i];
	    data.systemResponse[2 * i + 1] = curveArray[2 * i + 1];
	}
		
	// set the index at which t = 0 if it lies within the window
	//
	data.zero = zero;

	// set flag to inform data is ready to be calculated
	//
	data.inputDataReady = true;
    }

    /**
     * set the total capacity of the input data to preform the FFT on
     *
     * @param d data object containing the input samples
     */
    public void setCapacity (Data d) {
	
	d.numsamples = currLimit;
	d.capacity = 1;
	d.fftExponent = 0;
	d.maxsamples = curveArray.length;

	while(d.capacity < datarect.width) {
	    d.capacity = d.capacity << 1;
	    d.fftExponent += 1;
	}
	d.initializeArrays();
    }

    /**
     * method sets the updateFlag causing a repaint of the canvas
     *
     * @return true
     */
    public boolean updateScreen() {
	
	// set the updateFlag
	//
	updateFlag = true;
	
	// exit gracefully
	//
	return true;
    }

    /**
     * method draws the output response of the sysnal on the screen
     *
     * @return true
     */
    public boolean drawResponse() {

	// cheak to make sure that the curveArray has memory allocated
	// to it and that the currLimit is with in range
	//
	if (curveArray == null) {
	    return true;
	}
	if ((currLimit <= 0) || (currLimit > curveArray.length / 2)) {
	    currLimit = 1;
	}

        // attach the dataSet to the graph
        //
        dataSet = this.loadDataSet(curveArray, currLimit);

        // attach the data sets the both axis
        //
        xAxis.attachDataSet(dataSet);
        yAxis.attachDataSet(dataSet);
    
        // setup properties of the dataSet
        //
        dataSet.linestyle = 1;
        dataSet.linecolor = Color.blue;
    
        // exit gracefully
        //
        return true;
    }
    
    /**
     * This method is the body of the thread.  it will update the
     * display area when necessary.  the necessity is based on the
     * condition of the updateFlag
     */
    public void run() {

	// loop until the thread is terminated
        //
        while (phaseRespThread != null) {

	    // sleep for an appropriate number of milliseconds
            //
	    try {
		phaseRespThread.sleep(THREAD_SLEEP_TIME);
	    }
            catch (InterruptedException e) { }

            // update the screen if necessary
            //
            if (updateFlag == true) {

		// draw the output response
		//
		this.detachDataSets();
		this.drawResponse();
		repaint();
	
		// reset the update flag
		//
		updateFlag = false;
	    }
	}
    }

    /**
     * This method starts or restarts the threaded object.
     */
    public void start() {
    
	// see if a new thread needs to be created
        //
        if (phaseRespThread == null) {
	    phaseRespThread = new Thread(this);
	}
    
	// see if thread isAlive to determine if it is running
	//
        if (phaseRespThread.isAlive() == false) {

	    // start the thread
            //
            phaseRespThread.start();
	}
    }

    /**
     * This method stops the threaded object.
     */
    public void stop() {
    
	// set the applet thread to null
        //
        phaseRespThread = null;
    }
}

//
// end of file
